/*
/*  
 * GSM.h - Libary for using the GSM and GPS-Modul on the GSM/GPRS/GPS-Shield Rev.7
 * Initialization code and constants for talking to the SCI16IS750 are taken from the original
 * library from antrax datentechnik gmbh (http://antrax.de)
 * 
 * Libary is set to work with the arduino mega and debugs on Serial0.
 * This can be changed by changing defines in GPS.h
*/

#ifndef GSM_h
#define GSM_h
#include <Arduino.h>
#include "State_Machine.h"

// which serial port to use? mega supports Serial, Serial1 ... Serial4
#define GSM_SERIAL Serial1

// undefine to avoid debugging
#define GSM_DEBUG_SERIAL Serial // where to put debug messages? needs to be setup/started by the main programm itself!

#ifdef GSM_DEBUG_SERIAL
	#define DEBUG_MSG(text)  {if(_debug != 0){GSM_DEBUG_SERIAL.println(text);}}
#else
	#define DEBUG_MSG(text) ;
#endif

#define GSM_SERIAL_BAUD 9600

// pin settings
#define GSM_POWER_PIN 7
#define MAX_RETRY_AT_COMMAND 3

class GSM
  {     
    public:
      GSM();
	  GSM(byte debug);
	  // blockierend. kann 5-8 Sekunden dauern.
      bool initialize(char* pwd);
      bool read_gsm_buffer();
	  char * get_gsm_buffer();
	  void send_command(char * command);
	  // blockierend. kehrt nach erfolg oder 3 Fehlversuchen zurck
	  bool send_SMS(char * phone_number, char * message);
	  // blockierend. kehrt nach antwort von GSM zurck.
      char get_gsm_state(void);
private:
      byte _debug;
	  char gsm_buffer[80];
	  byte buffer_pos;
	  void clear_UART();
};
#endif